# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.conf import settings
from django.contrib import admin
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, JsonResponse
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from . import models
from . import utils


class IncreasingAdminMixin(object):
    def move_up_view(self, request, obj_id):
        instance = self.model.objects.get(pk=obj_id)
        return JsonResponse({'success': instance.up()})

    def move_down_view(self, request, obj_id):
        instance = self.model.objects.get(pk=obj_id)
        return JsonResponse({'success': instance.down()})

    def get_urls(self):
        app_label = self.model._meta.app_label
        model_name = self.model._meta.model_name
        urls = super(IncreasingAdminMixin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^api/(?P<obj_id>\d+)/move_up/$',
                self.admin_site.admin_view(self.move_up_view),
                name='{}_{}_increasing_move_up'.format(app_label, model_name),
            ),
            url(
                r'^api/(?P<obj_id>\d+)/move_down/$',
                self.admin_site.admin_view(self.move_down_view),
                name='{}_{}_increasing_move_down'.format(app_label, model_name),
            ),
        )
        return my_urls + urls

    def get_move_up_link(self, obj):
        app_label = self.model._meta.app_label
        model_name = self.model._meta.model_name
        href = reverse(
            'admin:{}_{}_increasing_move_up'.format(app_label, model_name),
            args=[obj.id]
        )
        return format_html("<a href='{}'>{}</a>".format(href, _('up')))
    get_move_up_link.short_description = _('up')

    def get_move_down_link(self, obj):
        app_label = self.model._meta.app_label
        model_name = self.model._meta.model_name
        href = reverse(
            'admin:{}_{}_increasing_move_down'.format(app_label, model_name),
            args=[obj.id]
        )
        return format_html("<a href='{}'>{}</a>".format(href, _('down')))
    get_move_down_link.short_description = _('down')

    def get_move_up_down_link(self, obj):
        app_label = self.model._meta.app_label
        model_name = self.model._meta.model_name
        href = reverse(
            'admin:{}_{}_increasing_move_up'.format(app_label, model_name),
            args=[obj.id]
        )
        up = "<a href='{}'>{}</a>".format(href, _('up'))
        href = reverse(
            'admin:{}_{}_increasing_move_down'.format(app_label, model_name),
            args=[obj.id]
        )
        down = "<a href='{}'>{}</a>".format(href, _('down'))
        return format_html("{} | {}".format(up, down))
    get_move_up_down_link.short_description = _('up/down')


class PluginInlinePageLanguage(admin.TabularInline):
    model = models.Plugin
    # template = 'admin/edit_inline/tabular_plugin.html'
    template = 'admin/edit_inline/tabular_plugin_tmp.html'
    extra = 0
    fields = (
        'get_edit_icon', 'get_context_icon', 'get_related_model_name',  # 'get_read_only_ordering',
        'name', 'language', 'site', 'page_language', 'slug', 'title',
        'is_jumper', 'ordering',  # 'get_ordering_actions',
    )
    readonly_fields = (
        'get_edit_icon', 'get_context_icon', 'get_related_model_name', 'name',
        'language', 'site', 'page_language', 'slug', 'title', 'get_read_only_ordering',
        'get_ordering_actions',
    )

    def get_context_icon(self, obj):
        modal_template = 'admin/txcms/child_plugins/plugin_context_modal.html'
        plugin_context_data = obj.get_related_model().get_context_data(None)
        plugin_context_data.update(obj.get_model_dict())
        context = dict()
        context['plugin_context_data'] = plugin_context_data
        context['obj_id'] = obj.id
        modals_html = render_to_string(modal_template, context)
        get_modal_link = """
        <div align="center">
           <a href='#PluginContextModal-{obj_id}' data-toggle="modal" id="getPluginContext-{obj_id}">{link_name}</a>
        </div>""".format(
            obj_id=obj.id,
            link_name=_('context'),
        )
        html_link_modals = modals_html + get_modal_link
        return html_link_modals
    get_context_icon.allow_tags = True
    get_context_icon.short_description = 'context'

    def get_read_only_ordering(self, obj):
        return obj.ordering
    get_read_only_ordering.short_description = 'ordering'
    get_read_only_ordering.allow_tags = True

    def get_ordering_actions(self, obj):
        up_icon = '<a href="#" class="actionUp">up</a>'
        down_icon = '<a href="#" class="actionDown">down</a>'

        qs_plugins = obj.page_language.plugins.all()
        ordering_list = list(set(qs_plugins.values_list('ordering', flat=True).order_by('ordering')))
        min_ordering = ordering_list[0]
        max_ordering = ordering_list[-1]

        if obj.ordering == min_ordering:
            up_icon = ''
        if obj.ordering == max_ordering:
            down_icon = ''

        if up_icon and down_icon:
            icon = '{up_icon} | {down_icon}'.format(
                up_icon=up_icon,
                down_icon=down_icon
            )
        elif up_icon:
            icon = up_icon
        elif down_icon:
            icon = down_icon
        else:
            icon = ''
        return icon
    get_ordering_actions.short_description = 'ordering actions'
    get_ordering_actions.allow_tags = True

    def get_edit_icon(self, obj):
        parent_app_label = self.parent_model._meta.app_label
        parent_model_name = self.parent_model._meta.model_name

        child_obj = getattr(obj, obj.plugin_type)
        app_label = child_obj._meta.app_label
        model_name = child_obj._meta.model_name
        page_language_id = child_obj.page_language_id

        # set reverse url
        reverse_url = reverse('admin:{}_{}_change_from_page_language'.format(
            app_label, model_name
        ), args=[obj.id, page_language_id])

        # set return url
        return_url = reverse('admin:{}_{}_change'.format(
            parent_app_label, parent_model_name
        ), args=[page_language_id])

        # set icon
        title = _("Edit {}".format(model_name))
        img = "<img src='{img_url}jmb/images/edit.gif' alt='{title}' title='{title}'/>".format(
            img_url=settings.STATIC_URL,
            title=title
        )

        edit_icon = "<a name='edit' href={reverse_url}?next={return_url}>{img}</a>".format(
            reverse_url=reverse_url,
            return_url=return_url,
            img=img,
        )
        return format_html(edit_icon)
    get_edit_icon.short_description = _('edit icon')


class PluginInlineSite(admin.TabularInline):
    model = models.Plugin
    # template = 'admin/edit_inline/tabular_plugin.html'
    template = 'admin/edit_inline/tabular_plugin_tmp.html'
    extra = 0
    fields = (
        'get_edit_icon', 'get_context_icon', 'get_related_model_name',  # 'get_read_only_ordering',
        'name', 'language', 'site', 'page_language', 'slug', 'title',
        'ordering',  # 'get_ordering_actions',
    )
    readonly_fields = (
        'get_edit_icon', 'get_context_icon', 'get_related_model_name', 'name',
        'language', 'site', 'page_language', 'slug', 'title', 'get_read_only_ordering',
        'get_ordering_actions',
    )

    def get_context_icon(self, obj):
        modal_template = 'admin/txcms/child_plugins/plugin_context_modal.html'
        plugin_context_data = obj.get_related_model().get_context_data(None)
        plugin_context_data.update(obj.get_model_dict())
        context = dict()
        context['plugin_context_data'] = plugin_context_data
        context['obj_id'] = obj.id
        modals_html = render_to_string(modal_template, context)
        get_modal_link = """
        <div align="center">
           <a href='#PluginContextModal-{obj_id}' data-toggle="modal" id="getPluginContext-{obj_id}">{link_name}</a>
        </div>""".format(
            obj_id=obj.id,
            link_name=_('context'),
        )
        html_link_modals = modals_html + get_modal_link
        return html_link_modals
    get_context_icon.allow_tags = True
    get_context_icon.short_description = 'context'

    def get_queryset(self, request):
        """
        return only plugins with site FK and with not page language FK
        """
        qs = super(PluginInlineSite, self).get_queryset(request)
        return qs.filter(page_language__isnull=True)

    def get_read_only_ordering(self, obj):
        return obj.ordering
    get_read_only_ordering.short_description = 'ordering'
    get_read_only_ordering.allow_tags = True

    def get_ordering_actions(self, obj):
        up_icon = '<a href="#" class="actionUp">up</a>'
        down_icon = '<a href="#" class="actionDown">down</a>'
        qs_plugins = obj.site.plugins.filter(site_id=1, page_language__isnull=True)
        ordering_list = list(set(qs_plugins.values_list('ordering', flat=True).order_by('ordering')))
        min_ordering = ordering_list[0]
        max_ordering = ordering_list[-1]

        if obj.ordering == min_ordering:
            up_icon = ''
        if obj.ordering == max_ordering:
            down_icon = ''

        if up_icon and down_icon:
            icon = '{up_icon} | {down_icon}'.format(
                up_icon=up_icon,
                down_icon=down_icon
            )
        elif up_icon:
            icon = up_icon
        elif down_icon:
            icon = down_icon
        else:
            icon = ''
        return icon
    get_ordering_actions.short_description = 'ordering actions'
    get_ordering_actions.allow_tags = True

    def get_edit_icon(self, obj):
        parent_app_label = self.parent_model._meta.app_label
        parent_model_name = self.parent_model._meta.model_name

        child_obj = getattr(obj, obj.plugin_type)
        app_label = child_obj._meta.app_label
        model_name = child_obj._meta.model_name
        site_id = child_obj.site_id

        # set reverse url
        reverse_url = reverse('admin:{}_{}_change_from_site'.format(
            app_label, model_name
        ), args=[obj.id, site_id])

        # set return url
        return_url = reverse('admin:{}_{}_change'.format(
            parent_app_label, parent_model_name
        ), args=[site_id])

        # set icon
        title = _("Edit {}".format(model_name))
        img = "<img src='{img_url}jmb/images/edit.gif' alt='{title}' title='{title}'/>".format(
            img_url=settings.STATIC_URL,
            title=title
        )

        edit_icon = "<a name='edit' href={reverse_url}?next={return_url}>{img}</a>".format(
            reverse_url=reverse_url,
            return_url=return_url,
            img=img,
        )
        return format_html(edit_icon)
    get_edit_icon.short_description = _('edit icon')


class GenericPluginAdmin(admin.ModelAdmin):
    """
    Create Customized plugin Admin
    """
    change_form_template = 'admin/txcms/change_form.html'

    readonly_fields = (
        'id', 'creator', 'last_modifier', 'date_create', 'date_last_modify',
        'plugin_type',
    )
    prepopulated_fields = {'slug': ('name',)}

    def __init__(self, model, admin_site):
        self.view_type = None
        self.page_language = None
        self.site = None
        self.language = None
        super(GenericPluginAdmin, self).__init__(model, admin_site)

    def get_urls(self):
        from django.conf.urls import url

        app_label = self.model._meta.app_label
        model_name = self.model._meta.model_name
        urls = super(GenericPluginAdmin, self).get_urls()
        my_urls = [
            url(
                r'^add_from_page_language/(?P<page_language_id>\d+)/$',
                self.admin_site.admin_view(self.add_view),
                {'view_type': 'from_page_language'},
                name='{}_{}_add_from_page_language'.format(app_label, model_name)),
            url(
                r'^change_from_page_language/(?P<object_id>\d+)/(?P<page_language_id>\d+)/$',
                self.admin_site.admin_view(self.change_view),
                {'view_type': 'from_page_language'},
                name='{}_{}_change_from_page_language'.format(app_label, model_name)),
            url(
                r'^add_from_site/(?P<site_id>\d+)/$',
                self.admin_site.admin_view(self.add_view),
                {'view_type': 'from_site'},
                name='{}_{}_add_from_site'.format(app_label, model_name)),
            url(
                r'^change_from_site/(?P<object_id>\d+)/(?P<site_id>\d+)/$',
                self.admin_site.admin_view(self.change_view),
                {'view_type': 'from_site'},
                name='{}_{}_change_from_site'.format(app_label, model_name)),
        ]
        return my_urls + urls

    def add_view(
            self, request, form_url='', extra_context=None,
            page_language_id=None, site_id=None, view_type=None):

        # set view type
        self.view_type = view_type

        # if page language exists set values
        if page_language_id:
            self.page_language = models.PageLanguage.objects.get(pk=page_language_id)
            self.site = self.page_language.page.site
            self.language = self.page_language.language

        # if site exists set values
        if site_id:
            self.site = Site.objects.get(pk=site_id)

        return super(GenericPluginAdmin, self).add_view(request, form_url, extra_context)

    def change_view(
            self, request, object_id, form_url='', extra_context=None,
            page_language_id=None, site_id=None, view_type=None):

        # set view type
        self.view_type = view_type

        # if page language exists set values
        if page_language_id:
            self.page_language = models.PageLanguage.objects.get(pk=page_language_id)
            self.site = self.page_language.page.site
            self.language = self.page_language.language

        # if site exists set values
        if site_id:
            self.site = Site.objects.get(pk=site_id)

        # add plugin context data to build menu
        extra_context = extra_context or {}
        plugin_instance = models.Plugin.objects.get(pk=object_id)
        plugin_context_data = plugin_instance.get_related_model().get_context_data(request)
        extra_context['plugin_context_data'] = plugin_context_data
        extra_context['collapse'] = True
        extra_context['plugin_context_data'].update(plugin_instance.get_model_dict())
        return super(GenericPluginAdmin, self).change_view(request, object_id, form_url, extra_context)

    def save_model(self, request, obj, form, change):
        if self.view_type == 'from_page_language':
            obj.language = self.language
            obj.site = self.site
            obj.page_language = self.page_language
        if self.view_type == 'from_site':
            obj.site = self.site
        super(GenericPluginAdmin, self).save_model(request, obj, form, change)

    def get_list_display(self, request):
        return self.model.get_fields()

    def get_fieldsets(self, request, obj=None):
        fields = self.model.get_fields()

        # change fieldsets if ADD or CHANGE from page language
        if self.view_type == 'from_page_language':
            fields.remove('language')
            fields.remove('site')
            fields.remove('page_language')

        # change fieldsets if ADD or CHANGE from site
        if self.view_type == 'from_site':
            fields.remove('site')
            fields.remove('page_language')

        return (
            (_('admin information'), {
                'fields': (
                    'creator', 'last_modifier',
                    'date_create', 'date_last_modify', 'plugin_type',
                ),
                'classes': ('collapse',),
            }),
            (_('plugin information'), {
                'fields': fields
            })
        )

    def response_post_save_add(self, request, obj):
        """
        Change response url after add if exist 'next' in querystring
        :param request:
        :param obj:
        :return:
        """
        if '_save' in request.POST:
            if request.GET.get('next', None):
                return HttpResponseRedirect(request.GET['next'])
        return super(GenericPluginAdmin, self).response_post_save_add(request, obj)

    def response_post_save_change(self, request, obj):
        """
        Change response url after change if exist 'next' in querystring
        :param request:
        :param obj:
        :return:
        """
        if '_save' in request.POST:
            if request.GET.get('next', None):
                return HttpResponseRedirect(request.GET['next'])
        return super(GenericPluginAdmin, self).response_post_save_change(request, obj)

    def response_change(self, request, obj):
        if '_continue' in request.POST:
            plugin_app_label = obj._meta.app_label
            plugin_model_name = obj._meta.model_name

            if obj.page_language:
                page_language_app_label = obj.page_language._meta.app_label
                page_language_model_name = obj.page_language._meta.model_name

                # set reverse url
                reverse_url = reverse('admin:{}_{}_change_from_page_language'.format(
                    plugin_app_label, plugin_model_name
                ), args=[obj.id, obj.page_language.id])

                # set return url
                return_url = reverse('admin:{}_{}_change'.format(
                    page_language_app_label, page_language_model_name
                ), args=[obj.page_language.id])
            else:
                site_app_label = obj.site._meta.app_label
                site_model_name = obj.site._meta.model_name

                # set reverse url
                reverse_url = reverse('admin:{}_{}_change_from_site'.format(
                    plugin_app_label, plugin_model_name
                ), args=[obj.id, obj.site.id])

                # set return url
                return_url = reverse('admin:{}_{}_change'.format(
                    site_app_label, site_model_name
                ), args=[obj.site.id])

            redirect_url = '{reverse_url}?next={return_url}'.format(
                reverse_url=reverse_url,
                return_url=return_url,
            )

            # insert the success message
            super(GenericPluginAdmin, self).response_change(request, obj)

            return HttpResponseRedirect(redirect_url)
        return super(GenericPluginAdmin, self).response_change(request, obj)

    def response_add(self, request, obj, post_url_continue=None):
        if '_continue' in request.POST:
            plugin_app_label = obj._meta.app_label
            plugin_model_name = obj._meta.model_name

            if obj.page_language:
                page_language_app_label = obj.page_language._meta.app_label
                page_language_model_name = obj.page_language._meta.model_name

                # set reverse url
                reverse_url = reverse('admin:{}_{}_change_from_page_language'.format(
                    plugin_app_label, plugin_model_name
                ), args=[obj.id, obj.page_language.id])

                # set return url
                return_url = reverse('admin:{}_{}_change'.format(
                    page_language_app_label, page_language_model_name
                ), args=[obj.page_language.id])
            else:
                site_app_label = obj.site._meta.app_label
                site_model_name = obj.site._meta.model_name

                # set reverse url
                reverse_url = reverse('admin:{}_{}_change_from_site'.format(
                    plugin_app_label, plugin_model_name
                ), args=[obj.id, obj.site.id])

                # set return url
                return_url = reverse('admin:{}_{}_change'.format(
                    site_app_label, site_model_name
                ), args=[obj.site.id])

            redirect_url = '{reverse_url}?next={return_url}'.format(
                reverse_url=reverse_url,
                return_url=return_url,
            )

            # insert the success message
            super(GenericPluginAdmin, self).response_add(request, obj, post_url_continue)

            return HttpResponseRedirect(redirect_url)
        return super(GenericPluginAdmin, self).response_add(request, obj, post_url_continue)


class GenericItemAdmin(admin.ModelAdmin):
    prepopulated_fields = {'slug': ('title',)}
    readonly_fields = (
        'id', 'creator', 'last_modifier', 'date_create', 'date_last_modify',
    )

    def get_list_display(self, request):
        return self.model.get_fields()

    def get_fieldsets(self, request, obj=None):
        fields = self.model.get_fields()
        return (
            (_('admin information'), {
                'fields': (
                    'creator', 'last_modifier',
                    'date_create', 'date_last_modify',
                ),
                'classes': ('collapse',),
            }),
            (_('plugin information'), {
                'fields': fields
            })
        )


class SiteAdmin(admin.ModelAdmin):
    change_form_template = 'admin/txcms/change_form.html'
    fieldsets = (
        (_('general admin'), {
            'fields': (
                ('domain', 'name',),
            )
        }),
    )
    list_display = ('domain', 'name',)
    inlines = [PluginInlineSite]

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = {}
        plugin_urls = []
        for plugin_name, plugin_dict in utils.get_related_models().items():
            plugin_url = {}
            try:
                plugin_url['name'] = _("{}".format(plugin_dict['verbose_name']))
                reverse_url = reverse(
                    'admin:{app_label}_{model_name}_add_from_site'.format(
                        app_label=plugin_dict['app_label'], model_name=plugin_dict['model_name']
                    ),
                    kwargs={'site_id': object_id},
                )
                next_url = reverse('admin:sites_site_change', args=[object_id])
                plugin_url['url'] = "{}?next={}".format(
                    reverse_url, next_url
                )
                plugin_urls.append(plugin_url)
            except:
                continue
        extra_context['add_urls'] = plugin_urls
        return super(SiteAdmin, self).change_view(request, object_id, form_url, extra_context)

    class Media:
        js = ('admin/js/plugin_inline/plugin_up_down.js',)


class PageAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                ('creator', 'date_create',), ('last_modifier', 'date_last_modify',)
            ),
            'classes': ('collapse',)
        }),
        (_('general admin'), {
            'fields': (
                'ordering', 'name', 'site',
            )
        }),
    )
    list_display = ('name', 'site', 'ordering',)
    list_editable = ('ordering',)
    readonly_fields = (
        'creator', 'date_create', 'last_modifier', 'date_last_modify',
    )


class NavAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                ('creator', 'date_create',), ('last_modifier', 'date_last_modify',)
            ),
            'classes': ('collapse',)
        }),
        (_('general admin'), {
            'fields': (
                'name', 'slug',
            )
        }),
    )
    list_display = ('name', 'slug',)
    prepopulated_fields = {'slug': ('name',)}


class PageLanguageAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                ('creator', 'date_create',), ('last_modifier', 'date_last_modify',)
            ),
            'classes': ('collapse',)
        }),
        (_('general admin'), {
            'fields': (
                ('status', 'ordering',), 'page', ('name', 'slug',), 'language',
                'is_homepage', 'navs', 'template_name', 'redirect_to',
                ('is_404', 'is_500',),
            )
        }),
        (_('meta admin'), {
            'fields': (
                'title', 'author',
                'meta_description', 'meta_keywords',
                'ga_code',  'google_site_verification',
            ),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'ordering', 'status',
        'is_homepage', 'get_nav_names', 'name', 'slug', 'is_redirect_to',
        'language', 'template_name', 'page',
    )
    list_editable = ('ordering', 'status',)
    filter_horizontal = ('navs',)
    inlines = [PluginInlinePageLanguage]
    list_display_csv = ('status', 'name', 'slug', 'title',)
    prepopulated_fields = {'slug': ('name',)}
    readonly_fields = (
        'creator', 'date_create', 'last_modifier', 'date_last_modify',
    )

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = {}
        plugin_urls = []
        for plugin_name, plugin_dict in utils.get_related_models().items():
            plugin_url = {}
            try:
                plugin_url['name'] = _("{}".format(plugin_dict['verbose_name']))
                reverse_url = reverse(
                    'admin:{app_label}_{model_name}_add_from_page_language'.format(
                        app_label=plugin_dict['app_label'], model_name=plugin_dict['model_name']
                    ),
                    kwargs={'page_language_id': object_id},
                )
                next_url = reverse('admin:txcms_pagelanguage_change', args=[object_id])
                plugin_url['url'] = "{}?next={}".format(
                    reverse_url, next_url
                )
                plugin_urls.append(plugin_url)
            except:
                continue
        extra_context['add_urls'] = plugin_urls
        return super(PageLanguageAdmin, self).change_view(request, object_id, form_url, extra_context)

    class Media:
        js = ('admin/js/plugin_inline/plugin_up_down.js',)


class PluginAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                ('creator', 'date_create',), ('last_modifier', 'date_last_modify',)
            ),
            'classes': ('collapse',)
        }),
        (_('general admin'), {
            'fields': (
                ('status', 'ordering',), ('name', 'slug',),
            )
        }),
    )
    list_display = (
        'status', 'plugin_type', 'name', 'language',
        'site', 'page_language', 'slug', 'title',
    )
    list_editable = ('status',)
    list_filter = ('language', 'site', 'page_language')
    readonly_fields = (
        'creator', 'date_create', 'last_modifier', 'date_last_modify',
    )

    def get_change_url(self, obj):
        app_label = obj._meta.app_label
        object_name = obj._meta.object_name.lower()
        if obj.get_related_model:
            app_label = obj.get_related_model()._meta.app_label
            object_name = obj.get_related_model()._meta.object_name.lower()
        return reverse(
            'admin:{}_{}_change'.format(app_label, object_name),
            args=(obj.pk,)
        )

    def get_edit_icon(self, obj, querystring=''):
        app_label = obj._meta.app_label
        object_name = obj._meta.object_name.lower()
        reverse_url = reverse('admin:{}_{}_changelist'.format(app_label, object_name))
        title = _("Edit {}".format(object_name))
        img = "<img src='{img_url}jmb/images/edit.gif' alt='{title}' title='{title}'/>".format(
            img_url=settings.STATIC_URL,
            title=title
        )

        edit_icon = "<a name='edit' href={reverse_url}?next={return_url}>{img}</a>".format(
            reverse_url=self.get_change_url(obj),
            return_url=reverse_url,
            img=img,
        )
        return format_html(edit_icon)
    get_edit_icon.short_description = _('edit icon')

    def has_add_permission(self, request):
        return False

    def changelist_view(self, request, extra_context=None):
        extra_context = {}
        plugin_urls = []
        for plugin_name, plugin_dict in utils.get_related_models().items():
            plugin_url = {}
            try:
                plugin_url['name'] = _("{}".format(plugin_dict['verbose_name']))
                reverse_url = reverse("admin:{}_{}_add".format(
                    plugin_dict['app_label'], plugin_dict['model_name']
                ))
                next_url = reverse("admin:txcms_plugin_changelist")
                plugin_url['url'] = "{}?next={}".format(
                    reverse_url, next_url
                )
                plugin_urls.append(plugin_url)
            except:
                continue
        extra_context['add_urls'] = plugin_urls
        return super(PluginAdmin, self).changelist_view(request, extra_context)


admin.site.register(models.Page, PageAdmin)
admin.site.register(models.Nav, NavAdmin)
admin.site.register(models.PageLanguage, PageLanguageAdmin)
admin.site.register(models.Plugin, PluginAdmin)

# unregister / register site model
admin.site.unregister(Site)
admin.site.register(Site, SiteAdmin)


# register all related plugin
for cls in models.Plugin.get_related_models():
    if not cls._meta.abstract:
        admin.site.register(cls, GenericPluginAdmin)


# register all related item
for cls in models.AbstractItemModel.get_related_models():
    if not cls._meta.abstract:
        admin.site.register(cls, GenericItemAdmin)
