# -*-coding: utf-8-*-
from __future__ import unicode_literals
from six import python_2_unicode_compatible

from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel


@python_2_unicode_compatible
class IncreasingModel(models.Model):
    ordering = models.IntegerField(
        blank=True, null=True,
        verbose_name=_('ordering'),
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.ordering is None:
            brothers = self.brothers()
            if brothers:
                ordering = brothers.last().ordering
                # se il fratello ha un valore impostato incremento
                self.ordering = ordering + 1 if ordering is not None else 0
            else:
                self.ordering = 0
        super(IncreasingModel, self).save(*args, **kwargs)

    def brothers_and_me(self):
        increasing_fields = getattr(self.__class__, 'icreasing_fields')
        filters = {}
        for field in increasing_fields:
            obj = self
            attrs = field.split('__')
            for attr in attrs:
                obj = getattr(obj, attr)
            filters[field] = obj
        return self._default_manager.filter(**filters)

    def brothers(self):
        return self.brothers_and_me().exclude(pk=self.pk)

    def up(self):
        brothers = self.brothers().filter(ordering__lt=self.ordering)
        if brothers.count():
            # scambio i valori 'ordering'
            ordering = self.ordering
            brother = brothers.last()
            self.ordering = brother.ordering
            self.save()
            brother.ordering = ordering
            brother.save()
            return True
        return False

    def down(self):
        brothers = self.brothers().filter(ordering__gt=self.ordering)
        if brothers.count():
            # scambio i valori 'ordering'
            ordering = self.ordering
            brother = brothers.first()
            self.ordering = brother.ordering
            self.save()
            brother.ordering = ordering
            brother.save()
            return True
        return False


@python_2_unicode_compatible
class Page(DateModel, UserModel, StatusModel, IncreasingModel):
    icreasing_fields = ('site',)
    name = models.CharField(
        max_length=255,
        verbose_name=_('page'),
    )
    site = models.ForeignKey(
        Site,
        related_name='pages',
        verbose_name=_('site'),
    )

    class Meta:
        verbose_name = _("page")
        verbose_name_plural = _("pages")
        ordering = ('ordering', 'name',)
        unique_together = (('name', 'site',),)

    def __str__(self):
        return "{}: {}".format(self.site, self.name)


@python_2_unicode_compatible
class Nav(DateModel, UserModel):
    """
    Model used to aggregate page links.
    Example: 'navbar', 'footer', 'footer_box_first', etc.
    """
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_('slug'),
    )

    class Meta:
        verbose_name = _("menu aggregator")
        verbose_name_plural = _("menu aggregators")
        ordering = ('name',)

    def __str__(self):
        return "{}".format(self.name)


@python_2_unicode_compatible
class PageLanguage(DateModel, UserModel, IncreasingModel):
    icreasing_fields = ('language', 'page__site')
    page = models.ForeignKey(
        Page,
        related_name='page_languages',
        verbose_name=_('page'),
    )
    status = models.IntegerField(
        choices=settings.THUX_TXCMS_PAGELANGUAGE_STATUS_CODES,
        default=1,
        verbose_name=_('Status'),
        null=True
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=255,
        verbose_name=_('slug'),
    )
    language = models.CharField(
        max_length=2,
        choices=settings.LANGUAGES,
        verbose_name=_('language'),
    )
    is_homepage = models.BooleanField(
        default=False,
        verbose_name=_('is homepage'),
    )
    navs = models.ManyToManyField(
        # used to insert 'language page' in different menu aggregators: 'navbar', 'footer' etc.
        Nav,
        related_name='page_languages',
        blank=True,
        verbose_name=_('navs'),
    )
    template_name = models.CharField(
        max_length=255,
        choices=settings.THUX_TXCMS_PAGE_TEMPLATE_NAME_CHOICES,
        verbose_name=_('template name'),
    )
    redirect_to = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('redirect to an absolute or relative url')
    )
    title = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('title'),
    )
    author = models.CharField(
        max_length=255,
        blank=True,
        default=settings.THUX_TXCMS_PAGE_AUTHOR_DEFAULT,
        verbose_name=_('author'),
    )
    meta_description = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('meta description'),
    )
    meta_keywords = models.CharField(
        max_length=255,
        blank=True,
        verbose_name=_('meta keywords'),
    )
    ga_code = models.CharField(
        max_length=10,
        blank=True,
        default=settings.THUX_TXCMS_PAGE_GA_CODE_DEFAULT,
        verbose_name=_('ga code'),
    )
    google_site_verification = models.CharField(
        max_length=10,
        blank=True,
        default=settings.THUX_TXCMS_PAGE_GS_VERIFICATION_DEFAULT,
        verbose_name=_('google site verification'),
    )
    is_404 = models.BooleanField(
        default=False,
        verbose_name=_('is 404'),
    )
    is_500 = models.BooleanField(
        default=False,
        verbose_name=_('is 500'),
    )

    class Meta:
        verbose_name = _("page language")
        verbose_name_plural = _("page languages")
        ordering = ('page__site', 'language', 'ordering', 'name',)
        unique_together = (('language', 'slug',),)

    def __str__(self):
        return "{}: {}".format(self.name, self.language)

    def save(self, *args, **kwargs):
        if self.is_homepage:
            # modifico le altre pagine 'homepage'
            for pagelang in PageLanguage.objects.filter(
                language=self.language, page__site=self.page.site
            ).exclude(pk=self.pk):
                pagelang.is_homepage = False
                pagelang.save()
        super(PageLanguage, self).save(*args, **kwargs)

        if self.is_500:
            other_page_page_languages = PageLanguage.objects.filter(
                language=self.language, page__site=self.page.site
            ).exclude(pk=self.pk)
            for page_language in other_page_page_languages:
                if page_language.is_500:
                    page_language.is_500 = False
                    page_language.save()

        if self.is_404:
            other_page_page_languages = PageLanguage.objects.filter(
                language=self.language, page__site=self.page.site
            ).exclude(pk=self.pk)
            for page_language in other_page_page_languages:
                if page_language.is_404:
                    page_language.is_404 = False
                    page_language.save()

    def get_absolute_url(self):
        if self.is_homepage:
            return '/'
        else:
            return "/{}/".format(self.slug)

    def get_nav_names(self):
        return ", ".join([nav.name for nav in self.navs.all()])
    get_nav_names.short_description = 'navs'
    get_nav_names.admin_order_field = 'navs'

    def get_jumpers(self):
        return self.plugins.filter(is_jumper=True)

    def is_redirect_to(self):
        return True if self.redirect_to else False
    is_redirect_to.boolean = True


@python_2_unicode_compatible
class GenericModel(models.Model):
    """
    Return all model fields
    """
    @classmethod
    def get_fields(cls):
        exclude = (
            'creator', 'last_modifier',
            'date_create', 'date_last_modify', 'plugin_type',
            'plugin_ptr',
        )
        fields = []
        for field in cls._meta.fields:
            if field.name not in exclude:
                fields.append(field.name)
        for field in cls._meta.many_to_many:
            if field.name not in exclude:
                fields.append(field.name)
        return fields

    class Meta:
        abstract = True


@python_2_unicode_compatible
class Plugin(DateModel, UserModel, StatusModel, OrderedModel, GenericModel):
    @classmethod
    def get_related_models(cls):
        """
        Return all inherited models
        """
        objs = cls._meta.get_all_related_objects()
        return [x.related_model for x in objs]

    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_('name'),
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_('slug'),
    )
    language = models.CharField(
        max_length=2,
        choices=settings.LANGUAGES,
        verbose_name=_('language'),
    )
    site = models.ForeignKey(
        Site,
        related_name='plugins',
        verbose_name=_('site'),
    )
    page_language = models.ForeignKey(
        PageLanguage,
        blank=True, null=True,
        related_name='plugins',
        verbose_name=_('page language'),
    )
    plugin_type = models.CharField(
        max_length=255,
        verbose_name=_('plugin type'),
    )
    is_jumper = models.BooleanField(
        default=False,
        verbose_name=_('is jumper'),
    )

    class Meta:
        verbose_name = _('plugin')
        verbose_name_plural = _('plugins')
        ordering = ('ordering', 'title',)

    def __str__(self):
        return "{}: {}".format(self.page_language, self.title)

    def save(self, *args, **kwargs):
        if not self.plugin_type:
            self.plugin_type = self._meta.model_name
        super(Plugin, self).save(*args, **kwargs)

    def get_context_data(self, request):
        return {}

    def get_language(self):
        return self.page.language

    def get_related_model(self):
        """
        Return child model instance
        """
        return getattr(self, self.plugin_type)

    def get_related_model_name(self):
        """
        Return child model verbose_name
        """
        return self.get_related_model()._meta.verbose_name_raw

    def get_model_dict(self):
        model_dict = self.get_related_model().__dict__.copy()
        for key in model_dict.keys():
            if key.startswith('_'):
                model_dict.pop(key)
        return model_dict

    def get_related_model_serialized_data(self):
        """
        Return serialized data from related model
        """
        related_model = self.get_related_model()
        if hasattr(related_model, 'get_serialized_data'):
            return related_model.get_serialized_data()
        raise AttributeError("Create 'get_serialized_data' in {}".format(self.get_related_model_name()))


class AbstractItemMixin(object):
    @classmethod
    def get_related_models(cls):
        """
        Return all inherited models
        """
        return cls.__subclasses__()

    @classmethod
    def get_fields(cls):
        """
        Return all model fields
        """
        exclude = (
            'creator', 'last_modifier', 'date_create', 'date_last_modify',
        )
        fields = []
        for field in cls._meta.fields:
            if field.name not in exclude:
                fields.append(field.name)
        return fields


@python_2_unicode_compatible
class AbstractItemModel(DateModel, UserModel, StatusModel, OrderedModel, AbstractItemMixin):
    title = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('title'),
    )
    slug = models.SlugField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.title