# coding: utf-8
from collections import OrderedDict

from django.conf import settings
from django.db.models import Q
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseServerError, HttpResponseRedirect
from django.shortcuts import get_object_or_404, Http404
from django.template import loader
from django.utils import translation
from django.views.generic import View, TemplateView

from thux.txcms.models import PageLanguage

from . import models
from . import utils


class PageLanguageMixin(object):
    def __init__(self):
        self.site = None
        self.page_language = None
        self.template_name = None
        self.language = None
        self.plugins = models.Plugin.objects.none()
        super(PageLanguageMixin, self).__init__()

    def get_language(self, **kwargs):
        if 'language' in kwargs:
            return kwargs['language']
        if '_language' in self.request.session:
            return self.request.session.get('_language')
        return settings.LANGUAGE_CODE

    def get_site(self):
        return get_object_or_404(models.Site, id=settings.SITE_ID)

    def get_page_language(self, **kwargs):
        slug = kwargs.get('page_language_slug', None)
        page_language = get_object_or_404(models.PageLanguage, slug=slug, **{
            'language': self.language, 'page__site': self.site,
            'is_homepage': False, 'status__gt': 0
        })
        self.check_draft_page(page_language)
        return page_language

    def check_draft_page(self, page_language):
        """Check if the user is logged in and is superuser"""
        if page_language.status > 1 and not self.request.user.is_superuser:
            raise Http404('No %PageLanguage matches the given query.')

    def get_template_names(self):
        return self.page_language.template_name

    def get_menu(self):
        return models.PageLanguage.objects.filter(status=1, language=self.language, navs__isnull=False)

    def get_plugins_qs(self):
        return models.Plugin.objects.filter(status=1, language=self.language)

    def get_all_plugins(self):
        """
        Return all plugins
        """
        new_plugins = OrderedDict()
        plugins = self.plugins.filter(
            Q(site=self.site, page_language__isnull=True) | Q(page_language=self.page_language)
        )
        for plugin in plugins:
            child_plugin = getattr(plugin, plugin.plugin_type, None)
            if child_plugin:
                for key, value in child_plugin.get_context_data(self.request).items():
                    setattr(child_plugin, key, value)
                new_plugins[plugin.slug] = child_plugin
            else:
                new_plugins[plugin.slug] = plugin
        return new_plugins

    def get_site_plugins(self):
        """
        Return only site plugins
        """
        new_plugins = OrderedDict()
        plugins = self.plugins.filter(
            site=self.site, page_language__isnull=True
        )
        for plugin in plugins:
            child_plugin = getattr(plugin, plugin.plugin_type, None)
            if child_plugin:
                for key, value in child_plugin.get_context_data(self.request).items():
                    setattr(child_plugin, key, value)
                new_plugins[plugin.slug] = child_plugin
            else:
                new_plugins[plugin.slug] = plugin
        return new_plugins

    def get_pagelanguage_plugins(self):
        """
        Return only page language plugins
        """
        new_plugins = OrderedDict()
        plugins = self.plugins.filter(
            page_language=self.page_language
        )
        for plugin in plugins:
            child_plugin = getattr(plugin, plugin.plugin_type, None)
            if child_plugin:
                for key, value in child_plugin.get_context_data(self.request).items():
                    setattr(child_plugin, key, value)
                new_plugins[plugin.slug] = child_plugin
            else:
                new_plugins[plugin.slug] = plugin
        return new_plugins

    def get_page_jumpers(self):
        """
        Return all the Page Jumpers in the page
        """
        page_plugins = self.get_pagelanguage_plugins()
        jumper_plugins = []
        for name, plugin in page_plugins.iteritems():
            if plugin.is_jumper and plugin.template_name:
                jumper_plugins.append(plugin)
        return jumper_plugins

    def get_all_page_jumpers(self):
        """
        Return all the Page Jumpers of all pages of session language
        """
        new_plugins = OrderedDict()
        plugins = self.plugins.filter(
            Q(site=self.site, page_language__isnull=False)
        )
        for plugin in plugins:
            child_plugin = getattr(plugin, plugin.plugin_type, None)
            if child_plugin:
                for key, value in child_plugin.get_context_data(self.request).items():
                    setattr(child_plugin, key, value)
                new_plugins[plugin.slug] = child_plugin
            else:
                new_plugins[plugin.slug] = plugin
        jumper_plugins = []
        for name, plugin in new_plugins.iteritems():
            if plugin.is_jumper and plugin.template_name:
                jumper_plugins.append(plugin)
        return jumper_plugins

    def get_navs(self):
        return utils.get_navs(self.language, self.request.user.is_superuser)

    def dispatch(self, request, *args, **kwargs):
        self.language = self.get_language(**kwargs)
        self.site = self.get_site()
        self.page_language = self.get_page_language(**kwargs)
        if self.page_language.redirect_to:
            return HttpResponseRedirect(self.page_language.redirect_to)
        return super(PageLanguageMixin, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        self.plugins = self.get_plugins_qs()

        context = super(PageLanguageMixin, self).get_context_data(**kwargs)
        context.update(
            menu=self.get_menu(),
            navs=self.get_navs(),
            page=self.page_language,
            page_jumpers=self.get_page_jumpers(),
            all_page_jumpers=self.get_all_page_jumpers(),
            plugins=self.get_all_plugins(),
            site_plugins=self.get_site_plugins(),
            page_plugins=self.get_pagelanguage_plugins(),
        )
        return context


class PageLanguageView(PageLanguageMixin, TemplateView):
    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class HomePageLanguageView(PageLanguageView):
    def get_page_language(self, **kwargs):
        page_language = get_object_or_404(models.PageLanguage, **{
            'language': self.language, 'page__site': self.site,
            'is_homepage': True, 'status__gt': 0
        })
        self.check_draft_page(page_language)
        return page_language


class Error400(PageLanguageView):
    def get(self, request, *args, **kwargs):
        template = loader.get_template(self.get_page_language().template_name)
        return HttpResponseNotFound(
            content=template.render(self.get_context_data()),
            content_type='text/html; charset=utf-8',
            status=404,
        )

    def get_page_language(self, **kwargs):
        try:
            page_language = models.PageLanguage.objects.get(is_404=True)
            return page_language
        except models.PageLanguage.DoesNotExist:
            raise Http404('Page language no found: 404')


class Error500(PageLanguageView):
    def get(self, request, *args, **kwargs):
        template = loader.get_template(self.get_page_language().template_name)
        return HttpResponseServerError(
            content=template.render(self.get_context_data()),
            content_type='text/html; charset=utf-8',
            status=500,
        )

    def get_page_language(self, **kwargs):
        try:
            page_language = models.PageLanguage.objects.get(is_500=True)
            return page_language
        except models.PageLanguage.DoesNotExist:
            raise Http404('Page language no found: 500')


class SitemapView(TemplateView):
    template_name = "sitemap.html"
    host = settings.BASE_URL
    content_type = "text/xml; charset=utf-8"

    def build_pages(self):
        active_pages = PageLanguage.objects.filter(
            status=1, is_404=False, is_500=False
        )
        pages = []
        for page in active_pages:
            current_page = {
                'loc': None,
                'lastmod': None,
                'changefreq': None,
            }
            # page URLS
            page_url = page.get_absolute_url()
            abs_url = settings.BASE_URL + page_url
            current_page['loc'] = abs_url
            # Date last modified
            date_mod = page.date_last_modify
            date_mod = str(date_mod.date())
            current_page['lastmod'] = date_mod
            # Change frequency
            changefreq = settings.THUX_TXCMS_SITEMAP_CHANGEFREQ
            current_page['changefreq'] = changefreq
            pages.append(current_page)
        return pages

    def get_context_data(self, **kwargs):
        context = super(SitemapView, self).get_context_data(**kwargs)
        context['pages'] = self.build_pages()
        return context


class RobotsView(View):
    def get_robots_body(self):
        robots_lines = [": ".join(line) for line in settings.THUX_TXCMS_ROBOTS_BODY]
        robots_body = '\n'.join(robots_lines)
        return robots_body

    def get(self, request, *args, **kwargs):
        return HttpResponse(self.get_robots_body(), content_type='text/plain; charset=utf-8')
